﻿using OSGeo.OGR;
using System;
using System.Globalization;

namespace GDALDemoUpdate
{
    class Config
    {
        public string DataSource { get; set; }
        public string LayerName { get; set; }
        public double Lng { get; set; }
        public double Lat { get; set; }

        public static Config FromArgsArray(string[] args, NumberFormatInfo nfi)
        {
            string dataSource = args[0];
            string layerName = args[1];
            string[] coords = args[2].Split(new char[] { ';' }, StringSplitOptions.RemoveEmptyEntries);
            double lng = 0, lat = 0.0;
            if (!(coords.Length >= 2 && double.TryParse(coords[0], NumberStyles.Number, nfi, out lng) && double.TryParse(coords[1], NumberStyles.Number, nfi, out lat)))
            {
                throw new ApplicationException("Lng & Lat coords not specifed");
            }
            return new Config()
            {
                DataSource = dataSource,
                LayerName = layerName,
                Lat = lat,
                Lng = lng
            };
        }
    }

    class Program
    {
        private const int GEOM_FIELD_IDX = 0;
        private const string COUNTRY_ATTR_NAME = "name";
        private const double COORDS_OFFSET = 20.0;

        private static NumberFormatInfo nfi = new NumberFormatInfo() { CurrencyDecimalSeparator = "." };

        static void Main(string[] args)
        {
            if (args.Length < 3)
            {
                Console.WriteLine("Usage: GDALDemoUpdate <datasource> <layername> <lng;lat>");
                PauseAndExit(0);
            }
            Config config = null;
            try
            {
                config = Config.FromArgsArray(args, nfi);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                PauseAndExit(1);
            }          

            GdalConfiguration.ConfigureOgr();

            DataSource ds = Ogr.Open(config.DataSource, 1);
            if (ds == null)
            {
                Console.WriteLine("DataSource open failed");
                PauseAndExit(1);
            }

            Layer layer = ds.GetLayerByName(config.LayerName);
            if (layer == null)
            {
                Console.WriteLine("Layer not found");
                ds.Dispose();
                PauseAndExit(1);
            }

            Feature feature = GetFeatureByCoordinates(layer, config.Lng, config.Lat);
            if (feature != null)
            {
                Console.WriteLine($"Modifying: {feature.GetFieldAsString(COUNTRY_ATTR_NAME)}");
                UpdateFeature(feature);
                layer.SetFeature(feature);
                ds.SyncToDisk();
                feature.Dispose();
            }
            else
            {
                Console.WriteLine($"No feature found");
            }

            layer.Dispose();
            ds.Dispose();

            PauseAndExit(0);
        }

        static Feature GetFeatureByCoordinates(Layer layer, double x, double y)
        {
            string wkt = $"POINT({x.ToString(nfi)} {y.ToString(nfi)})";
            Geometry filter = Ogr.CreateGeometryFromWkt(ref wkt, layer.GetLayerDefn().GetGeomFieldDefn(GEOM_FIELD_IDX).GetSpatialRef());
            layer.SetSpatialFilter(filter);
            Feature feature = layer.GetNextFeature();
            filter.Dispose();
            return feature;

        }

        static void UpdateFeature(Feature feature)
        {
            string newName = $"{feature.GetFieldAsString(COUNTRY_ATTR_NAME)} is so amazing!";
            Console.WriteLine($"New name: {newName}");
            feature.SetField(COUNTRY_ATTR_NAME, newName);
            UpdateGeometry(feature.GetGeometryRef(), "  ");
        }

        static void UpdateGeometry(Geometry geom, string padding)
        {
            Console.WriteLine($"{padding}GeometryName:{geom.GetGeometryName()}, GeometryType:{geom.GetGeometryType()}");
            int pointCount = geom.GetPointCount();
            for (int j = 0; j < pointCount; j++)
            {
                double[] points = new double[3];
                geom.GetPoint(j, points);
                for (int i = 0; i < points.Length - 1; i++)
                {
                    points[i] += COORDS_OFFSET;
                    Console.WriteLine($"{padding}New coords: [x:{points[0]}, y:{points[1]}]");
                }
                geom.SetPoint(j, points[0], points[1], points[2]);
            }

            int geomCount = geom.GetGeometryCount();
            for (int i = 0; i < geomCount; i++)
            {
                UpdateGeometry(geom.GetGeometryRef(i), padding + "  ");
            }
        }

        static void PauseAndExit(int errCode)
        {
            Console.ReadLine();
            Environment.Exit(errCode);
        }
    }
}